from invenio_records_resources.services import RecordService
from invenio_records_resources.services import RecordService as InvenioRecordService


class CommonService(RecordService):
    """CommonRecord service."""



import click
import tqdm
import yaml
from flask.cli import with_appcontext
from invenio_access.permissions import system_identity
from invenio_db import db
from invenio_search import current_search
from invenio_search.cli import search_version_check

from nr_metadata.documents.proxies import current_service
from nr_metadata.documents.records.api import DocumentsRecord


@click.group(name="nr-metadata.documents")
def group():
    pass


@group.command()
@click.option("--recreate", is_flag=True, help="Drop and create the index")
@click.option("--queue", is_flag=True, help="Use celery to perform bulk indexing")
@with_appcontext
@search_version_check
def reindex(recreate, queue, *args, **kwargs):
    if recreate:
        index = DocumentsRecord.index._name
        status = list(current_search.delete(index_list=[index]))
        print_status(f"Delete index {index}: ", status)
        status = list(current_search.create(index_list=[index]))
        print_status(f"Create index {index}: ", status)

    # reindex all
    id_generator = (
        x[0]
        for x in db.session.query(DocumentsRecord.model_cls.id).filter(
            DocumentsRecord.model_cls.is_deleted.is_(False)
        )
    )

    if queue:
        current_service.indexer.bulk_index(
            # iterator of ids
            id_generator
        )
        print("Records send to the queue")
    else:
        ids = list(id_generator)
        for rec_id in tqdm.tqdm(ids, "Indexing records"):
            record = DocumentsRecord.get_record(rec_id)
            current_service.indexer.index(record)
        current_service.indexer.refresh()
        print(f"Indexing finished, indexed {len(ids)} records")


def print_status(title, status):
    print(title)
    for status_val in status:
        for k, v in status_val[1].items():
            print(f"    {k}: {v}")


@group.command()
@click.argument("data-path", type=click.Path(file_okay=True, dir_okay=False))
@with_appcontext
@search_version_check
def load(data_path, *args, **kwargs):
    with open(data_path) as f:
        for doc in tqdm.tqdm(yaml.load_all(f, yaml.SafeLoader), desc="Saving data"):
            current_service.create(system_identity, doc)



import operator
from copy import deepcopy
from functools import reduce

from deepmerge import always_merger
from invenio_records.dumpers import SearchDumperExt


def getFromDict(dataDict, mapList):
    return reduce(operator.getitem, mapList, dataDict)


class MultilingualDumper(SearchDumperExt):
    """CommonRecord search dumper."""

    def dump(self, record, data):
        paths = []
        SUPPORTED_LANGS = ["cs", "en"]

        for path in paths:
            new_elements = {}
            record2 = record
            path_array = path.split("/")
            path_array2 = []

            for x in path_array:
                path_array2.append(x)

            path_array2.pop(0)
            path_array2 = path_array2[:-1]

            for x in path_array2:
                record2 = record2[x]
            path_array.pop(0)
            multilingual_element = getFromDict(record, path_array)

            for rec in multilingual_element:
                if rec["lang"] in SUPPORTED_LANGS:
                    el_name = path_array[-1] + "_" + rec["lang"]
                    always_merger.merge(new_elements, {el_name: rec["value"]})

            always_merger.merge(record2, new_elements)
        data.update(deepcopy(dict(record)))
        return data

    def load(self, record, data):
        paths = []
        SUPPORTED_LANGS = ["cs", "en"]
        for path in paths:
            record2 = record
            path_array = path.split("/")
            path_array2 = []
            for x in path_array:
                path_array2.append(x)

            path_array2.pop(0)
            path_array2 = path_array2[:-1]

            for x in path_array2:
                record2 = record2[x]

            path_array.pop(0)
            multilingual_element = getFromDict(record, path_array)
            for rec in multilingual_element:
                if rec["lang"] in SUPPORTED_LANGS:
                    el_name = path_array[-1] + "_" + rec["lang"]
                    del record2[el_name]
        return data



import marshmallow as ma
from edtf import Date as EDTFDate
from edtf import Interval as EDTFInterval
from invenio_records_resources.services.records.schema import (
    BaseRecordSchema as InvenioBaseRecordSchema,
)
from invenio_vocabularies.services.schema import i18n_strings
from marshmallow import ValidationError
from marshmallow import fields as ma_fields
from marshmallow import validate as ma_validate
from marshmallow_utils import fields as mu_fields
from marshmallow_utils import schemas as mu_schemas
from marshmallow_utils.fields import edtfdatestring as mu_fields_edtf
from oarepo_runtime.i18n.schema import I18nSchema
from oarepo_runtime.ui import marshmallow as l10n
from oarepo_runtime.validation import validate_date
from oarepo_vocabularies.services.schemas import HierarchySchema

from nr_metadata.common.services.records.schema import (
    AdditionalTitlesSchema,
    NRAccessRightsVocabularySchema,
    NRAffiliationVocabularySchema,
    NRAuthorityRoleVocabularySchema,
    NRAuthoritySchema,
    NRContributorSchema,
    NRCountryVocabularySchema,
    NREventSchema,
    NRExternalLocationSchema,
    NRFunderVocabularySchema,
    NRFundingReferenceSchema,
    NRGeoLocationPointSchema,
    NRGeoLocationSchema,
    NRItemRelationTypeVocabularySchema,
    NRLanguageVocabularySchema,
    NRLocationSchema,
    NRRelatedItemSchema,
    NRResourceTypeVocabularySchema,
    NRSeriesSchema,
    NRSubjectCategoryVocabularySchema,
    NRSubjectSchema,
)
from nr_metadata.schema.identifiers import (
    NRAuthorityIdentifierSchema,
    NRObjectIdentifierSchema,
    NRSystemIdentifierSchema,
)


class NRDegreeGrantorSchema(ma.Schema):
    """NRDegreeGrantorSchema schema."""

    _id = ma_fields.String(data_key="id", attribute="id")
    title = i18n_strings
    hierarchy = ma_fields.Nested(lambda: HierarchySchema())
    _version = ma_fields.String(data_key="@v", attribute="@v")


class NRThesisSchema(ma.Schema):
    """NRThesisSchema schema."""

    dateDefended = ma_fields.String(validate=[validate_date("%Y-%m-%d")])
    defended = ma_fields.Boolean()
    degreeGrantors = ma_fields.List(ma_fields.Nested(lambda: NRDegreeGrantorSchema()))
    studyFields = ma_fields.List(ma_fields.String())


class NRDocumentMetadataSchema(ma.Schema):
    """NRDocumentMetadataSchema schema."""

    thesis = ma_fields.Nested(lambda: NRThesisSchema())
    collection = ma_fields.String()
    title = ma_fields.String()
    additionalTitles = ma_fields.List(
        ma_fields.Nested(lambda: AdditionalTitlesSchema())
    )
    creators = ma_fields.List(ma_fields.Nested(lambda: NRAuthoritySchema()))
    contributors = ma_fields.List(ma_fields.Nested(lambda: NRContributorSchema()))
    resourceType = ma_fields.Nested(lambda: NRResourceTypeVocabularySchema())
    dateAvailable = ma_fields.String(
        validate=[mu_fields_edtf.EDTFValidator(types=(EDTFDate,))]
    )
    dateModified = ma_fields.String(
        validate=[mu_fields_edtf.EDTFValidator(types=(EDTFDate,))]
    )
    subjects = ma_fields.List(ma_fields.Nested(lambda: NRSubjectSchema()))
    publishers = ma_fields.List(ma_fields.String())
    subjectCategories = ma_fields.List(
        ma_fields.Nested(lambda: NRSubjectCategoryVocabularySchema())
    )
    languages = ma_fields.List(ma_fields.Nested(lambda: NRLanguageVocabularySchema()))
    notes = ma_fields.List(ma_fields.String())
    abstract = ma_fields.List(ma_fields.Nested(lambda: I18nSchema()))
    methods = ma_fields.List(ma_fields.Nested(lambda: I18nSchema()))
    technicalInfo = ma_fields.List(ma_fields.Nested(lambda: I18nSchema()))
    rights = ma_fields.List(ma_fields.Nested(lambda: NRAccessRightsVocabularySchema()))
    accessRights = ma_fields.Nested(lambda: NRAccessRightsVocabularySchema())
    relatedItems = ma_fields.List(ma_fields.Nested(lambda: NRRelatedItemSchema()))
    fundingReferences = ma_fields.List(
        ma_fields.Nested(lambda: NRFundingReferenceSchema())
    )
    version = ma_fields.String()
    geoLocations = ma_fields.List(ma_fields.Nested(lambda: NRGeoLocationSchema()))
    accessibility = ma_fields.List(ma_fields.Nested(lambda: I18nSchema()))
    series = ma_fields.List(ma_fields.Nested(lambda: NRSeriesSchema()))
    externalLocation = ma_fields.Nested(lambda: NRExternalLocationSchema())
    originalRecord = ma_fields.String()
    objectIdentifiers = ma_fields.List(
        ma_fields.Nested(lambda: NRObjectIdentifierSchema())
    )
    systemIdentifiers = ma_fields.List(
        ma_fields.Nested(lambda: NRSystemIdentifierSchema())
    )
    events = ma_fields.List(ma_fields.Nested(lambda: NREventSchema()))


class NRDocumentRecordSchema(InvenioBaseRecordSchema):
    """NRDocumentRecordSchema schema."""

    metadata = ma_fields.Nested(lambda: NRDocumentMetadataSchema())
    created = ma_fields.String(validate=[validate_date("%Y-%m-%d")], dump_only=True)
    updated = ma_fields.String(validate=[validate_date("%Y-%m-%d")], dump_only=True)
